var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _FacadeTransport_url;
import { makeid } from './utils';
import { GameAccount, GameBundle, PlayerProfile, RegistrationAccount, RegistrationWithGames, ServerAccount, } from '@race-foundation/sdk-core';
import { deserialize } from '@race-foundation/borsh';
export class FacadeTransport {
    constructor(url = 'http://localhost:12002') {
        _FacadeTransport_url.set(this, void 0);
        __classPrivateFieldSet(this, _FacadeTransport_url, url, "f");
    }
    async createGameAccount(wallet, params) {
        const walletAddr = wallet.walletAddr;
        const gameAddr = makeid(16);
        const data = [...params.data];
        const ix = { walletAddr, gameAddr, ...params, data };
        await this.sendInstruction('create_account', ix);
        return gameAddr;
    }
    closeGameAccount(wallet, params) {
        throw new Error('Method not implemented.');
    }
    deposit(wallet, params) {
        throw new Error('Method not implemented.');
    }
    vote(wallet, params) {
        throw new Error('Method not implemented.');
    }
    publishGame(wallet, params) {
        throw new Error('Method not implemented.');
    }
    createRegistration(wallet, params) {
        throw new Error('Method not implemented.');
    }
    registerGame(wallet, params) {
        throw new Error('Method not implemented.');
    }
    unregisterGame(wallet, params) {
        throw new Error('Method not implemented.');
    }
    async listTokens() {
        return [
            {
                name: "USD Coin",
                symbol: "USDC",
                decimals: 6,
                icon: "https://raw.githubusercontent.com/solana-labs/token-list/main/assets/mainnet/EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v/logo.png",
                addr: "FACADE_USDC",
            },
            {
                name: "Tether USD",
                symbol: "USDT",
                decimals: 6,
                icon: "https://raw.githubusercontent.com/solana-labs/token-list/main/assets/mainnet/Es9vMFrzaCERmJfrF4H2FYD4KCoNkY11McCe8BenwNYB/logo.svg",
                addr: "FACADE_USDT",
            },
            {
                name: "Native Token",
                symbol: "NATIVE",
                decimals: 9,
                icon: "https://arweave.net/SH106hrChudKjQ_c6e6yd0tsGUbFIScv2LL6Dp-LDiI",
                addr: "FACADE_NATIVE"
            },
            {
                name: "Race Protocol",
                symbol: "RACE",
                decimals: 9,
                icon: "https://raw.githubusercontent.com/NutsPokerTeam/token-list/main/assets/mainnet/RACE5fnTKB9obGtCusArTQ6hhdNXAtf3HarvJM17rxJ/logo.svg",
                addr: "FACADE_RACE",
            },
        ];
    }
    async createPlayerProfile(wallet, params) {
        const playerAddr = wallet.walletAddr;
        const ix = { playerAddr, ...params };
        await this.sendInstruction('create_profile', ix);
    }
    async join(wallet, params) {
        const playerAddr = wallet.walletAddr;
        const ix = { playerAddr, ...params };
        await this.sendInstruction('join', ix);
    }
    async getGameAccount(addr) {
        const data = await this.fetchState('get_account_info', [addr]);
        if (data === undefined)
            return undefined;
        return deserialize(GameAccount, data);
    }
    async getGameBundle(addr) {
        const data = await this.fetchState('get_game_bundle', [addr]);
        if (data === undefined)
            return undefined;
        return deserialize(GameBundle, data);
    }
    async getPlayerProfile(addr) {
        const data = await this.fetchState('get_profile', [addr]);
        if (data === undefined)
            return undefined;
        return deserialize(PlayerProfile, data);
    }
    async getServerAccount(addr) {
        const data = await this.fetchState('get_server_info', [addr]);
        if (data === undefined)
            return undefined;
        return deserialize(ServerAccount, data);
    }
    async getRegistration(addr) {
        const data = await this.fetchState('get_registration_info', [addr]);
        if (data === undefined)
            return undefined;
        return deserialize(RegistrationAccount, data);
    }
    async getRegistrationWithGames(addr) {
        const data = await this.fetchState('get_registration_info', [addr]);
        if (data === undefined)
            return undefined;
        const regAccount = deserialize(RegistrationAccount, data);
        const promises = regAccount.games.map(async (g) => {
            return await this.getGameAccount(g.addr);
        });
        const games = await Promise.all(promises);
        return new RegistrationWithGames({
            ...regAccount,
            games
        });
    }
    async getToken(addr) {
        return {
            addr,
            icon: '',
            name: 'Fake Token',
            symbol: 'FT',
            decimals: 0,
        };
    }
    async fetchBalances(walletAddr, tokenAddrs) {
        let ret = new Map();
        for (const addr of tokenAddrs) {
            const data = await this.fetchState("get_balance", [walletAddr, addr]);
            if (data !== undefined) {
                const view = new DataView(data.buffer);
                const balance = view.getBigUint64(0, true);
                ret.set(addr, balance);
            }
        }
        return ret;
    }
    async getNft(_addr) {
        return undefined;
    }
    async listNfts(_) {
        return [];
    }
    async sendInstruction(method, ix) {
        const reqData = JSON.stringify({
            jsonrpc: '2.0',
            method,
            id: makeid(16),
            params: [ix],
        }, (_key, value) => (typeof value === 'bigint' ? Number(value) : value));
        const resp = await fetch(__classPrivateFieldGet(this, _FacadeTransport_url, "f"), {
            method: 'POST',
            body: reqData,
            headers: {
                'Content-Type': 'application/json',
            },
        });
        if (!resp.ok) {
            throw new Error('Failed to send instruction: ' + reqData);
        }
    }
    async fetchState(method, params) {
        const reqData = JSON.stringify({
            jsonrpc: '2.0',
            method,
            id: makeid(16),
            params,
        });
        const resp = await fetch(__classPrivateFieldGet(this, _FacadeTransport_url, "f"), {
            method: 'POST',
            body: reqData,
            headers: {
                'Content-Type': 'application/json',
            },
        });
        if (!resp.ok) {
            throw new Error('Failed to fetch data at :' + params);
        }
        const { result } = await resp.json();
        if (result !== null) {
            return Uint8Array.from(result);
        }
        else {
            return undefined;
        }
    }
}
_FacadeTransport_url = new WeakMap();
//# sourceMappingURL=facade-transport.js.map